import { useSchemaGraph, useSchemaSettings, useSearchResultSchema, useSessionCache } from '@graphpolaris/shared/lib/data-access/store';
import { AlgorithmToLayoutProvider, AllLayoutAlgorithms, LayoutFactory } from '@graphpolaris/shared/lib/graph-layout';
import { schemaExpandRelation, schemaGraphology2Reactflow } from '@graphpolaris/shared/lib/schema/schema-utils';
import { SmartBezierEdge, SmartStepEdge, SmartStraightEdge } from '@tisoap/react-flow-smart-edge';
import { useEffect, useMemo, useRef, useState } from 'react';
import ReactFlow, { Edge, Node, ReactFlowInstance, ReactFlowProvider, useEdgesState, useNodesState } from 'reactflow';
import 'reactflow/dist/style.css';
import { wsSchemaRequest } from '@graphpolaris/shared/lib/data-access/broker/wsSchema';
import { ConnectionDragLine, ConnectionLine } from '@graphpolaris/shared/lib/querybuilder/pills';
import { Button } from '../../components/buttons';
import ControlContainer from '../../components/controls';
import { wsGetStates } from '../../data-access';
import { toSchemaGraphology } from '../../data-access/store/schemaSlice';
import NodeEdge from '../pills/edges/node-edge';
import SelfEdge from '../pills/edges/self-edge';
import { EntityNode } from '../pills/nodes/entity/entity-node';
import { RelationNode } from '../pills/nodes/relation/relation-node';
import { SchemaDialog } from './schemaDialog';
import { ExpandMore, ArrowRight } from '@mui/icons-material';
import { Tooltip, TooltipContent, TooltipProvider, TooltipTrigger } from '../../components/tooltip';
import { FormControl } from '../../components';
import Input from '../../components/inputs';

interface Props {
  content?: string;
  auth?: boolean;
}

const onInit = (reactFlowInstance: ReactFlowInstance) => {
  setTimeout(() => reactFlowInstance.fitView(), 100);
};

const nodeTypes = {
  entity: EntityNode,
  relation: RelationNode,
};
const edgeTypes = {
  nodeEdge: NodeEdge,
  selfEdge: SelfEdge,
  bezier: SmartBezierEdge,
  connection: ConnectionLine,
  straight: SmartStraightEdge,
  step: SmartStepEdge,
};

export const Schema = (props: Props) => {
  const settings = useSchemaSettings();
  const searchResults = useSearchResultSchema();
  const [toggleSchemaSettings, setToggleSchemaSettings] = useState(false);
  const [nodes, setNodes, onNodesChange] = useNodesState([] as Node[]);
  const [edges, setEdges, onEdgesChange] = useEdgesState([] as Edge[]);
  const [firstUserConnection, setFirstUserConnection] = useState<boolean>(true);
  const [auth, setAuth] = useState(props.auth);
  const [expanded, setExpanded] = useState<boolean>(false);

  const reactFlowInstanceRef = useRef<ReactFlowInstance | null>(null);

  // In case the schema is updated
  const schemaGraph = useSchemaGraph();
  const schemaGraphology = useMemo(() => toSchemaGraphology(schemaGraph), [schemaGraph]);
  const layout = useRef<AlgorithmToLayoutProvider<AllLayoutAlgorithms>>();

  function updateLayout() {
    const layoutFactory = new LayoutFactory();
    layout.current = layoutFactory.createLayout(settings.layoutName);
  }

  const fitView = () => {
    if (reactFlowInstanceRef.current) {
      reactFlowInstanceRef.current.fitView();
    }
  };

  useEffect(() => {
    updateLayout();
    sessionStorage.setItem('firstUserConnection', firstUserConnection.toString());
  }, []);

  useEffect(() => {
    setAuth(props.auth);
  }, [props.auth]);

  async function layoutGraph() {
    if (schemaGraphology === undefined || schemaGraphology.order == 0) {
      setNodes([]);
      setEdges([]);
      return;
    }

    updateLayout();
    const expandedSchema = schemaExpandRelation(schemaGraphology);
    await layout.current?.layout(expandedSchema);
    const schemaFlow = schemaGraphology2Reactflow(expandedSchema, settings.connectionType, settings.animatedEdges);
    setNodes(schemaFlow.nodes);
    setEdges(schemaFlow.edges);
  }

  useEffect(() => {
    layoutGraph();
  }, [schemaGraph, settings]);

  useEffect(() => {
    setNodes((nds) =>
      nds.map((node) => ({
        ...node,
        selected: searchResults.includes(node.id) || searchResults.includes(node.data.label),
      })),
    );
  }, [searchResults]);

  return (
    <div className="schema-panel w-full h-full flex flex-col justify-between">
      {nodes.length === 0 ? (
        <p className="text-sm">No Elements</p>
      ) : (
        <ReactFlowProvider>
          <div className="h-[calc(100%-.8rem)] w-full">
            <ReactFlow
              snapGrid={[10, 10]}
              snapToGrid
              onlyRenderVisibleElements={false}
              nodesDraggable={true}
              nodeTypes={nodeTypes}
              edgeTypes={edgeTypes}
              connectionLineComponent={ConnectionDragLine}
              onNodesChange={onNodesChange}
              onEdgesChange={onEdgesChange}
              nodes={nodes}
              edges={edges}
              onInit={(reactFlowInstance) => {
                reactFlowInstanceRef.current = reactFlowInstance;
                onInit(reactFlowInstance);
              }}
              proOptions={{ hideAttribution: true }}
            ></ReactFlow>
          </div>
        </ReactFlowProvider>
      )}
      <div className="relative bottom-0">
        <div className="w-full py-0 px-2 bg-secondary-50 cursor-pointer border-y flex" onClick={() => setExpanded(!expanded)}>
          <Button
            size="2xs"
            variant="ghost"
            iconComponent={expanded ? <ExpandMore /> : <ArrowRight />}
            onClick={() => setExpanded(!expanded)}
          />
          <span className="text-xs font-semibold text-secondary-800 ml-2">Schema settings</span>
        </div>
        {expanded && (
          <div className="h-full w-full overflow-y-auto">
            <SchemaDialog open={toggleSchemaSettings} onClose={() => setToggleSchemaSettings(false)} />
          </div>
        )}
      </div>
    </div>
  );
};

export default Schema;
